/**************************************************************************************************|
|* Copyright 2016 Hycon Technology, Corp.                                                         *|
|* http://www.hycontek.com/                                                                       *|
|* Description:                                                                                   *|
|*                                                                                                *|
|**************************************************************************************************/

/**************************************************************************************************|
|*  Includes                                                                                      *|
|**************************************************************************************************/
#include <hy11s14sfr.h>    // Peripheral Register Memory Mapping
#include <ADC.h>
#include <CLK.h>

/**************************************************************************************************|
|*  Static Function                                                                               *|
|**************************************************************************************************/

/**************************************************************************************************|
|*  Generic Peripheral Library                                                                    *|
|**************************************************************************************************/

/*-------------------------------------------------------------------------------------------------|
|  Function: ADC_Open                                                                              |
|                                                                                                  |
|  Parameters:                                                                                     |
|         ck        - [in] MCKCN1_ADCCK_HSDCK   / MCKCN1_ADCCK_LSCK                                |
|         cks       - [in] MCKCN1_ADCS_DIV1     / MCKCN1_ADCS_DIV2    / MCKCN1_ADCS_DIV4           |
|                          MCKCN1_ADCS_DIV8     / MCKCN1_ADCS_DIV16   / MCKCN1_ADCS_DIV32          |
|                          MCKCN1_ADCS_DIV64    / MCKCN1_ADCS_DIV128                               |
|         inh       - [in] AINET1_INH_TPSH0     / AINET1_INH_TPSH1    / AINET1_INH_ADM             |
|                          AINET1_INH_HALF_VDDA / AINET1_INH_AI0      / AINET1_INH_AI4             |
|                          AINET1_INH_AI6       / AINET1_INH_AI2                                   |
|         inl       - [in] AINET1_INL_TPSL1     / AINET1_INL_TPSL0    / AINET1_INL_VSS             |
|                          AINET1_INL_OPO       / AINET1_INL_AI1      / AINET1_INL_AI5             |
|                          AINET1_INL_AI7       / AINET1_INL_AI3                                   |
|         vrh       - [in] AINET2_VRH_ACM       / AINET2_VRH_AI2                                   |
|                          AINET2_VRH_AI8       / AINET2_VRH_AI6                                   |
|         vrl       - [in] AINET2_VRL_VSS       / AINET2_VRL_AI3                                   |
|                          AINET2_VRL_AI9       / AINET2_VRL_AI5                                   |
|         adgn      - [in] ADCCN1_ADGN_1DIV4    / ADCCN1_ADGN_1DIV2   / ADCCN1_ADGN_1              |
|                          ADCCN1_ADGN_2        / ADCCN1_ADGN_4       / ADCCN1_ADGN_8              |
|                          ADCCN1_ADGN_16                                                          |
|         pgagn     - [in] ADCCN1_PGAGN_1       / ADCCN1_PGAGN_2                                   |
|                          ADCCN1_PGAGN_4       / ADCCN1_PGAGN_8                                   |
|         vrgn      - [in] ADCCN2_VREGN_DIV1    / ADCCN2_VREGN_DIV2                                |
|         dcset     - [in] ADCCN2_DCSET_0       / ADCCN2_DCSET_P1DIV4 / ADCCN2_DCSET_P1DIV2        |
|                          ADCCN2_DCSET_P3DIV4  / ADCCN2_DCSET_00     / ADCCN2_DCSET_N1DIV4        |
|                          ADCCN2_DCSET_N1DIV2  / ADCCN2_DCSET_N3DIV4                              |
|         osr       - [in] ADCCN3_OSR_256       / ADCCN3_OSR_512      / ADCCN3_OSR_1024            |
|                          ADCCN3_OSR_2048      / ADCCN3_OSR_4096     / ADCCN3_OSR_8192            |
|                          ADCCN3_OSR_16384     / ADCCN3_OSR_32768                                 |
|                                                                                                  |
|  Returns:                                                                                        |
|         None                                                                                     |
|                                                                                                  |
|  Description:                                                                                    |
|         This function is used to open the Watch Dog Timer(WDT)                                   |
|-------------------------------------------------------------------------------------------------*/
void ADC_Open(unsigned char ck,    //Selector of SD18 peripheral operating clock source
              unsigned char cks,   //Prescaler of SD18 peripheral operating frequency
              unsigned char inh,   //SI+ input signal selector
              unsigned char inl,   //SI- input signal selector
              unsigned char vrh,   //VR+ voltage signal selector
              unsigned char vrl,   //VR- voltage signal selector
              unsigned char adgn,  //AD amplification adjuster
              unsigned char pgagn, //PGA amplification adjuster
              unsigned char vrgn,  //VR amplification adjuster
              unsigned char dcset, //SI+/- bias adjuster
              unsigned char osr)  //SD18 over-sampling rate divider
{
    // ADC CLK setting //
    CLK_ADCCKDivSel(cks);
    CLK_ADCCKSel(ck);

    // ADC Init //
    ADC_AINConfig(inh,inl);             // Config ADC Input Channel
    ADC_VRINConfig(vrh,vrl);            // Config ADC Ref. Voltage Input Channel
    ADC_GainConfig(adgn,pgagn);         // Config PGA Gain and AD Gain
    ADC_VRGainSelect(vrgn);             // Config Ref. Gain and DC offset
    ADC_DCSetConfig(dcset);             // Config Ref. Gain and DC offset
    ADC_OSRConfig(osr);                 // Config Over sampling ratio
    ADC_Enable();
}

/*-------------------------------------------------------------------------------------------------|
|  Function: ADC_GetData                                                                           |
|                                                                                                  |
|  Parameters:                                                                                     |
|         None                                                                                     |
|                                                                                                  |
|  Returns:                                                                                        |
|         ADCData   - [in] Read the 24-bit data for ADCR                                           |
|                                                                                                  |
|  Description:                                                                                    |
|         This function is used to Read the Data for ADC                                           |
|-------------------------------------------------------------------------------------------------*/
long ADC_GetData(void)
{
    long ADCData;
    ((char *)(&ADCData))[0] = ADCRL;
    ((char *)(&ADCData))[1] = ADCRM;
    ((char *)(&ADCData))[2] = ADCRH;

    if (ADCData&0x800000L)
        ((char *)(&ADCData))[3] = 0xFF;
    else
        ((char *)(&ADCData))[3] = 0x00;
    return ADCData;
}

/**************************************************************************************************|
|*  Static Functions                                                                              *|
|**************************************************************************************************/
